package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import com.javamonkey.utils.CommandLineOptions;
import java.io.File;
import java.io.FileWriter;

/**
 * A command line tool for running tests.
 *
 * @version $Id$
 */
public class CommandLineTestRunner {

  public static void main(String[] argv) {
    CommandLineOptions opts = new CommandLineOptions();
    opts.addFlag("tee", "Should test results be printed to stdout.");
    opts.parseCommandLine(argv);

    if (opts.numberOfArguments() < 2) {
      fail("args: <test class name> <results file>");
    }

    String testClass       = opts.argument(0);
    String resultsFileName = opts.argument(1);

    File resultsFile = new File(resultsFileName);

    try {
      Class c = Class.forName(testClass);
      TestRunner runner = new TestRunner(c);
      runner.addTestListener(new Logger(new FileWriter(resultsFile)));
      if (opts.asBoolean("tee")) {
	runner.addTestListener(new Logger(System.out));
      }
      runner.run();

      if (runner.allTestsPassed()) {
	System.out.println(testClass + " passed all tests.");
	System.exit(0);
      } else {
	fail(
	  testClass + " failed " + runner.getFailures() + " of " + 
	  runner.getTotal() + " test cases.");
      }
    } catch (ClassNotFoundException cnfe) {
      resultsFile.delete();
      fail("Couldn't load " + testClass);
    } catch (Exception e) {
      resultsFile.delete();
      fail("Exception", e);
    }



  }

  private static void fail(String msg) {
    System.err.println(msg);
    System.exit(1);
  }

  private static void fail(String msg, Exception e) {
    System.err.println(msg);
    e.printStackTrace();
    System.exit(1);
  }


}











