package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import java.io.File;
import java.io.FileReader;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * A simple test that reads a file of test class names and runs them.
 */
public class FileTest extends BaseTest {

  private List tests = new ArrayList();
  
  public FileTest(File f) throws IOException {
    this(f.getName(), new FileReader(f));
  }

  public FileTest(String name, Reader in) throws IOException {
    BufferedReader br = new BufferedReader(in);
    String className = null;
    int testCasesCount = 0;
    while ((className = br.readLine()) != null) {
      try {
	Class c = Class.forName(className);
	Test t = (Test)c.newInstance();
	tests.add(t);
	testCasesCount += t.getNumberOfTestCases();
	t.addTestListener(this);
      } catch (ClassNotFoundException cnfe) {
	System.err.println("No test: " + className);
      } catch (InstantiationException ie) {
	System.err.println("Couldn't instantiate: " + className);
      } catch (IllegalAccessException iae) {
	System.err.println("Couldn't access constructor on: " + className);
      }
    }
    setName(name);
    setNumberOfTestCases(testCasesCount);
  }

  public void run() {
    Iterator i = tests.iterator();
    while (i.hasNext()) {
      Test t = (Test)i.next();
      t.run();
    }
  }


}
