package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.Writer;

/**
 * Test logger.
 */
public class Logger implements TestListener {
    
  /**
   * Where we write our output.
   */
  private PrintWriter out;
  
  /**
   * Make a logger that writes to the given Writer.
   */
  public Logger(Writer o) {
    if (o instanceof PrintWriter) {
      out = (PrintWriter)o;
    } else {
      out = new PrintWriter(o, true);
    }
  }
  
  /**
   * Make a logger that writes to the given stream.
   */
  public Logger(OutputStream s) {
    this(new OutputStreamWriter(s));
  }
  

  // =====================================================================
  // TestListener implementation

  public void testSucceeded(TestEvent evt) {
    log("PASS", evt);
  }
  
  public void testFailed(TestEvent evt) {
    log("FAIL", evt);
    logException(evt);
  }

  public void testSkipped(TestEvent evt) {
    log("SKIP", evt);
    logException(evt);
  }

  public void testAborted(TestEvent evt) {
    log("ABRT", evt);
    logException(evt);
  }

  // End TestListener implementation.
  // =====================================================================

  private void log(String label, TestEvent evt) {
    out.println(
      label + ": " + evt.getName() + messageString(evt.getMessage()));
  }

  private String messageString(String message) {
    return message == null ? "" : ": " + message;
  }

  private void logException(TestEvent evt) {
    Throwable t = evt.getThrowable();
    if (t != null) t.printStackTrace(out);
  }
}






