package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import javax.swing.UIManager;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.*;
import com.javamonkey.utils.Debug;
import java.beans.*;
import com.javamonkey.utils.CommandLineOptions;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;

/**
 * GUI console for running tests.
 *
 * @author Copyright 1999 Peter Seibel, All Rights Reserved.
 */
public class TestConsole implements TestListener {

  private JFrame frame;
  private JPanel panel;
  private GridBagLayout layout;
  private GridBagConstraints gbc;
  private JProgressBar progressBar;
  private ResultsPanel resultsPanel;
  private JMenu fileMenu;
  private JMenuBar menuBar;

  private Test test;

  private int successes;
  private int failures;
  
  public TestConsole() {
    UIManager.put("ProgressBar.selectionForeground", Color.black);
    UIManager.put("ProgressBar.selectionBackground", Color.black);
    frame().pack();
    frame().setVisible(true);
  }

  private GridBagLayout layout() {
    if (layout == null) {
      layout = new GridBagLayout();
    }
    return layout;
  }

  private GridBagConstraints gbc() {
    if (gbc == null) {
      gbc = new GridBagConstraints();
    }
    return gbc;
  }

  private JPanel panel() {
    if (panel == null) {
      panel = new JPanel(layout());

      gbc().fill = GridBagConstraints.BOTH;
      gbc().gridwidth = GridBagConstraints.REMAINDER;
      layout().setConstraints(progressBar(), gbc());
      panel.add(progressBar());

      panel.add(resultsPanel());
    }
    return panel;
  }


  private JProgressBar progressBar() {
    if (progressBar == null) {
      progressBar = new JProgressBar();
      progressBar.setStringPainted(true);
      progressBar.setForeground(Color.green);

    }
    return progressBar;
  }


  private ResultsPanel resultsPanel() {
    if (resultsPanel == null) {
      resultsPanel = new ResultsPanel();
    }
    return resultsPanel;
  }

  private JFrame frame() {
    if (frame == null) {
      frame = new JFrame("TestConsole");
      frame.setJMenuBar(menuBar());
      frame.addWindowListener(new WindowCloser());
      frame.getContentPane().add(panel());
    }
    return frame;
  }
    
  private JMenuBar menuBar() {
    if (menuBar == null) {
      menuBar = new JMenuBar();
      menuBar.add(fileMenu());
    }
    return menuBar;
  }
   
  private JMenu fileMenu() {
    if (fileMenu == null) {
      fileMenu = new JMenu("File");
      fileMenu.add(chooseTestAction());
      fileMenu.add(quitAction());
    }
    return fileMenu;
  }

  // Actions supported by this application

  private Action quitAction;
  private Action chooseTestAction;

  private Action quitAction() {
    if (quitAction == null) {
      quitAction = new AbstractAction("Quit") {
	public void actionPerformed(ActionEvent evt) {
	  System.exit(0);
	}
      };
    }
    return quitAction;
  }

  private Action chooseTestAction() {
    if (chooseTestAction == null) {
      chooseTestAction = new AbstractAction("Choose test") {
	public void actionPerformed(ActionEvent evt) {
	  final JDialog diag = new JDialog(frame(), "Choose test", true);

	  JButton ok = new JButton("Ok");
	  ok.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent ae2) {
	      diag.dispose();
	      // It doesn't seem like this should be necessary but
	      // without it the frame is often dorked up after you
	      // close the dialog.
	      frame.repaint();
	    }
	  });
	  Box box = new Box(BoxLayout.Y_AXIS);

	  JLabel label = new JLabel("Enter test class:");
	  JComboBox choices = new JComboBox(new Object[] {
	      "com.javamonkey.test.TestTest" });
	  choices.setEditable(true);
	  
	  label.setAlignmentX(0.5f);
	  choices.setAlignmentX(0.5f);
	  ok.setAlignmentX(0.5f);

  	  box.add(label);
	  box.add(choices);
	  box.add(ok);
	  diag.getContentPane().add(box);
	  
	  diag.pack();
	  diag.show();
	}
      };
      chooseTestAction.putValue(Action.NAME, "Choose test");
    }
    return chooseTestAction;
  }

  // =================================================================
  // TestListener implementation
    
  public void testSucceeded(TestEvent evt) {
    handleTestEvent(evt);
  }
  public void testFailed(TestEvent evt) {
    handleTestEvent(evt);
  }
  public void testSkipped(TestEvent evt) {
    handleTestEvent(evt);
  }
  public void testAborted(TestEvent evt) {
    handleTestEvent(evt);
  }

  // End TestListener implementation.
  // =================================================================

  private void handleTestEvent(final TestEvent evt) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
	switch (evt.getResult()) {
	case Test.SUCCESS:
	  successes++;
	  break;
	case Test.FAILURE:
	  failures++;
	  progressBar().setForeground(Color.red);
	  break;

	case Test.SKIPPED:
	case Test.ABORTED:
	  progressBar().setForeground(Color.red);
	  break;

	default:
	  Debug.panic("Illegal test result code: " + evt.getResult());
	}

	progressBar.setValue(successes + failures);
	resultsPanel.update();
      }
    });
  }
  
  /**
   * Run the given test, after attaching ourselves to it so we can
   * display the results.
   */
  private void runTest(Test t) {

    // Disconnect from the old test if we have one.
    if (test != null) test.removeTestListener(this);

    // Connect to the new one.
    test = t;
    test.addTestListener(this);

    // Get the number of test cases in the test
    SwingUtilities.invokeLater(new Runnable() {
	public void run() {
	    progressBar().setMaximum(test.getNumberOfTestCases());
	}
    });

    // And away we go ...
    test.run();
  }

  /**
   * Helper class to handle closing of main frame.
   */
  private class WindowCloser extends WindowAdapter {
    public void windowClosing(WindowEvent evt) {
      frame.dispose();
    }
    public void windowClosed(WindowEvent evt) {
      System.exit(0);
    }
  }

  /**
   * Results panel -- displays in text form the number of successes,
   * failures, skips, and aborts.
   */
  private class ResultsPanel extends JPanel {

    private JLabel successesLabel;
    private JLabel failuresLabel;

    ResultsPanel() {
      super(new GridLayout(2,2));
      add(new JLabel("Successes"));
      successesLabel = new JLabel(""+successes, SwingConstants.CENTER);
      add(successesLabel);

      add(new JLabel("Failures"));
      failuresLabel = new JLabel(""+failures, SwingConstants.CENTER);
      add(failuresLabel);
    }

    void update() {
      successesLabel.setText(""+successes);
      failuresLabel.setText(""+failures);
    }
  }

  public static void main(String[] argv) throws Exception {

    CommandLineOptions opts = new CommandLineOptions();
    String testName = argv[0];
    Test test = new FileTest(new File(testName));
    //    Test test = (Test)Class.forName(testName).newInstance();
//      getCommandLineOptionsFromTest(test, opts);
//      opts.parseCommandLine(argv);
//      setTestOptionsFromCommandLine(test, opts);

    TestConsole console = new TestConsole();
    
    System.out.println("Creating logger");
    test.addTestListener(new Logger(System.out));
    
    console.runTest(test);
  }

  private static void getCommandLineOptionsFromTest(
    Test t, CommandLineOptions opts)
    throws IntrospectionException
  {
    BeanInfo info = Introspector.getBeanInfo(t.getClass());
    PropertyDescriptor[] descs = info.getPropertyDescriptors();
    for (int i = 0; i < descs.length; i++) {
      PropertyDescriptor d = descs[i];
      if (d.getWriteMethod() == null) continue;
      opts.addVariable(d.getName(), "Needed by " + t.getClass().getName());
    }
  }

  private static void setTestOptionsFromCommandLine(Test t, CommandLineOptions opts)
    throws IntrospectionException
  {
    BeanInfo info = Introspector.getBeanInfo(t.getClass());
    PropertyDescriptor[] descs = info.getPropertyDescriptors();
    for (int i = 0; i < descs.length; i++) {
      PropertyDescriptor d = descs[i];
      Method setter = d.getWriteMethod();
      if (setter == null) continue;
      String sValue = opts.asString(d.getName());
      doSet(d, t, sValue);
    }
  }

  private static void doSet(PropertyDescriptor d, Test t, String sValue) {
    System.err.println("Setting " + d.getName() + " to " + sValue);
    Method setter = d.getWriteMethod();
    Class type = d.getPropertyType();
    Object[] args = new Object[] { coerce(sValue, type) };
    try {
      setter.invoke(t, args);
    } catch (InvocationTargetException ite) {
      ite.printStackTrace();
      ite.getTargetException().printStackTrace();
    } catch (IllegalArgumentException iae) {
      iae.printStackTrace();
    } catch (IllegalAccessException iae) {
      iae.printStackTrace();
    }
  }

  private static Object coerce(String value, Class type) {

    if (type.isPrimitive()) {
      type = getWrapperClass(type);
    }
    try {
      Constructor c = type.getConstructor(new Class[] { String.class });
      return c.newInstance(new Object[] {value});
    } catch (Exception e) {
      e.printStackTrace();
      return null;
    }

  }

  private static Class getWrapperClass(Class type) {
    if (type == Integer.TYPE) {
      return Integer.class;
    } else if (type == Boolean.TYPE) {
      return Boolean.class;
    } else if (type == Character.TYPE) {
      return Character.class;
    } else if (type == Byte.TYPE) {
      return Byte.class;
    } else if (type == Short.TYPE) {
      return Short.class;
    } else if (type == Float.TYPE) {
      return Float.class;
    } else if (type == Double.TYPE) {
      return Double.class;
    } else if (type == Long.TYPE) {
      return Long.class;
    } else {
      Debug.panic(type + " is not primitive"); return null;
    }
  }

  private static Test getTestTest(String[] argv) {

    // Get arguments off the command line.
    int iters = argv.length > 0 ? Integer.parseInt(argv[0]) : 100;
    int delay = argv.length > 1 ? Integer.parseInt(argv[1]) : 0;

    // Make a test that will exercise the console.
    return new TestTest(iters, delay);

  }

}










