package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import java.util.EventListener;

/**
 * Listener interface for TestEvents. Implementations of the Test
 * interface use this listener interface to communicate their results
 * to interested parties such as the TestConsole and the Logger.
 *
 * @see com.javamonkey.test.TestEvent
 */
public interface TestListener extends EventListener {

  /**
   * A test case has completed with success.
   * 
   * @param evt The TestEvent bearing the details of the result.
   */
  public void testSucceeded(TestEvent evt);
    
  /**
   * A test case has completed with failure.
   *
   * @param evt The TestEvent bearing the details of the result.
   */
  public void testFailed(TestEvent evt);

  /**
   * A test case has been skipped. This indicates that the test case
   * either was begun and could not complete with a meaningful result
   * or was not run at all.
   */
  public void testSkipped(TestEvent evt);
  
  /**
   * A test case has aborted. This indicates that the test case was
   * begun but not only completed 
   */
  public void testAborted(TestEvent evt);
  
}





 



