package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import javax.swing.*;
import java.util.Random;

/**
 * Test implementation for exercising the TestConsole.
 */
public class TestTest extends BaseTest implements Test {

  /**
   * Number of milliseconds to delay between test cases.
   */
  private int delayMillis;

  /**
   * Source of (pseudo)randomness.
   */
  private Random random = new Random();

  /**
   * Make an unitilialized TestTest. This constructor should only be
   * used by callers who are going to use the property setting methods
   * setNumberOfTestCases() and setDelayMillis() to configure the
   * test.
   */
   public TestTest() {}

  /**
   * Make a TestTest that will generate the given number of test
   * cases, pausing for the given number of millis.
   *
   * @param cases Number of test cases.
   * @param delayMillis Number of milliseconds to delay between test cases.
   */
  public TestTest(int cases, int delayMillis) {
    super(cases);
    this.delayMillis = delayMillis;
  }

  /**
   * Set the number of test results to generate.
   */
  public void setNumberOfTestCases(int num) {
    // super version is protected.
    super.setNumberOfTestCases(num);
  }

  /**
   * Set the number of milliseconds to delay between each test result.
   */
  public void setDelayMillis(int millis) {
    delayMillis = millis;
  }

 
  /**
   * Run for the given number of cases, randomly reporting success and
   * failure. Pause between each test case for delayMillis
   * milliseconds.
   */
  public void run() {  
    
    for (int i = 0; i < getNumberOfTestCases(); i++) {
      try {
	if (delayMillis > 0) Thread.sleep(delayMillis);
	String name = "test-"+i;
	boolean result = random.nextBoolean();
	if (random.nextBoolean()) {
	  reportSuccess(name, (String)null);
	} else {
	  reportFailure(name, (String)null);
	}
      } catch (InterruptedException ie) {
      }
    }
  }
}



