package com.javamonkey.utils.reflect;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import com.javamonkey.test.AutoTest;

/**
 * Unit test for com.javamonkey.utils.reflect.Reflector.
 *
 * @version $Id: //depot/macaca/tests/com/javamonkey/utils/reflect/ReflectorTest.java#2 $
 */
public class ReflectorTest extends AutoTest {

  private Reflector r;
  private Object o;
  private final static Object[] EMPTY = new Object[0];

  public ReflectorTest() {
    r = new Reflector();
    o = new TestObject();
  }

  public boolean testPrivateBooleanField() throws Exception {
    return expect(true, r.booleanField("privateBoolean", o));
  }

  public boolean testPrivateByteField() throws Exception {
    return expect(2, r.byteField("privateByte", o));
  }

  public boolean testPrivateCharField() throws Exception {
    return expect(3, r.charField("privateChar", o));
  }

  public boolean testPrivateDoubleField() throws Exception {
    return expect(4.0, r.doubleField("privateDouble", o));
  }

  public boolean testPrivateFloatField() throws Exception {
    return expect(5.0f, r.floatField("privateFloat", o));
  }

  public boolean testPrivateIntField() throws Exception {
    return expect(6, r.intField("privateInt", o));
  }

  public boolean testPrivateLongField() throws Exception {
    return expect(7L, r.longField("privateLong", o));
  }

  public boolean testPrivateShortField() throws Exception {
    return expect(8, r.shortField("privateShort", o));
  }


  public boolean testInvokePublicVoidNoArgs() {
    try {
      r.invoke("public_void_no_args", o, EMPTY);
      return success("ok.");
    } catch (Exception e) {
      return false;
    }
  }

  public boolean testInvokePublicIntNoArgs() {
    try {
      Integer i = (Integer)r.invoke("public_int_no_args", o, EMPTY);
      return i.intValue() == 10;
    } catch (Exception e) {
      return false;
    }
  }

  public boolean testInvokePublicIntString() {
    try {
      Integer i = (Integer)r.invoke("public_int_String", o, new String[] { "10" });
      return i.intValue() == 10;
    } catch (Exception e) {
      e.printStackTrace();
      return false;
    }
  }

  public boolean testInvokePrivateVoidNoArgs() {
    try {
      r.invoke("private_void_no_args", o, EMPTY);
      return success("ok.");
    } catch (Exception e) {
      return false;
    }
  }

  public boolean testInvokePrivateIntNoArgs() {
    try {
      Integer i = (Integer)r.invoke("private_int_no_args", o, EMPTY);
      return i.intValue() == 10;
    } catch (Exception e) {
      return false;
    }
  }

  public boolean testInvokePrivateIntString() {
    try {
      Integer i = (Integer)r.invoke("private_int_String", o, new String[] { "10" });
      return i.intValue() == 10;
    } catch (Exception e) {
      e.printStackTrace();
      return false;
    }
  }


  private boolean testNoMethodFoundException() {
    try {
      r.invoke("XXX", o, EMPTY);
      return false;
    } catch (NoMethodFoundException nmfe) {
      return success("ok.");
    } catch (Exception e) {
      e.printStackTrace();
      return false;
    }
  }

  public boolean testNoMethodFoundException2() {
    try {
      r.invoke("private_int_String", o, EMPTY);
      return false;
    } catch (NoMethodFoundException nmfe) {
      return success("ok.");
    } catch (Exception e) {
      e.printStackTrace();
      return false;
    }
  }

  public boolean testAmbiguousInvoke() {
    try {
      r.invoke("ambiguous", o, new Object[] { "aString" });
      return false;
    } catch (AmbiguousInvocationException aie) {
      return success("ok.");
    } catch (Exception e) {
      e.printStackTrace();
      return false;
    }
  }

  public boolean testAmbiguousInvoke2() {
    try {
      r.invoke("ambiguous2", o, new Object[] { "aString",
					       "anotherString" });
      return false;
    } catch (AmbiguousInvocationException aie) {
      return success("ok.");
    } catch (Exception e) {
      e.printStackTrace();
      return false;
    }
  }


  public void report(String label, boolean result) {
    System.out.println(label + " ... " + (result ? "ok." : "not ok."));
  }
}

class TestObject {

  private boolean privateBoolean = true;
  private byte privateByte = 2;
  private char privateChar = 3;
  private double privateDouble = 4.0;
  private float privateFloat = 5.0f;
  private int privateInt = 6;
  private long privateLong = 7L;
  private short privateShort = 8;

  public void public_void_no_args() {}
  public int public_int_no_args() { return 10; }
  public int public_int_String(String s) { return Integer.parseInt(s); }
  private void private_void_no_args() {}
  private int private_int_no_args() { return 10; }
  private int private_int_String(String s) { return Integer.parseInt(s); }
  public void ambiguous(String s) {}
  public void ambiguous(Object o) {}
  public void ambiguous2(Object o, String s) {}
  public void ambiguous2(String s, Object o) {}
}





